﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Globalization;
using CG2.Rendering;
using CG2.Mathematics;
using CG2.Modeling;
using CG2.Shading;
using CG2.Lighting;
//using CG2.Texturing;

namespace CG2.Ex09
{
    public partial class Form1 : Form
    {
        #region Properties

        World world = new World();
        CultureInfo provider = new CultureInfo("en-US");
        Camera camera;

        #endregion

        public Form1()
        {
            InitializeComponent();
            cbUseShadows.Checked = true;
            cbScene.SelectedIndex = 0;
            cbAntiAlias.SelectedIndex = 0;
            WriteValues();
        }

        public void InitSceneSphereMapping()
        {
            World world = new World();

            #region Shaders

            Shader blue = new Phong(new Vector4(81 / 255.0, 138 / 255.0, 214 / 255.0));
            Shader unsaturatedGreen = new Phong(new Vector4(86 / 255.0, 116 / 255.0, 86 / 255.0));

            //Shader checker = new Phong(new Vector4(81 / 255.0, 138 / 255.0, 214 / 255.0), new Vector4(0.3, 0.3, 0.3))
            //{
            //    DiffuseSampler = new TextureSampler("Textures/checker.png", null, true)
            //    {
            //        Mapping = new SphereMapping(new Vector4(0, -2, 2), Vector4.UnitX, Vector4.UnitY) { Scale = new Vector4(0.3, 0.3, 1) }
            //    }
            //};

            #endregion

            #region Models

            //world.Models.Add(new Sphere(checker, new Vector4(0, -2, -2), 6));
            world.Models.Add(new Plane(unsaturatedGreen, new Vector4(0, 0, -10), new Vector4(0, 0, 1)));

            #endregion

            #region Lights

            PointLight point = new PointLight()
            {
                Origin = new Vector4(10, 0, 6),
                Intensity = 1.0,
                Range = 70.0,
                LinearAttenuation = 0.2,
                QuadraticAttenuation = 0.5
            };
            world.Lights.Add(point);

            #endregion

            camera = new Camera(500, 400)
            {
                Position = new Vector4(20, 0, 10),
                Target = new Vector4(0, 0, 0),
                World = world,
                zNear = 0.01,
                zFar = 100.0,
            };
            WriteValues();
        }

        public void InitScenePlaneMapping()
        {
            //And after nearly a whole semester of hard work we can finally render ..... scene from wolf 3D? wtffffff
            World world = new World();

            #region Shaders

            Shader blue = new Phong(new Vector4(81 / 255.0, 138 / 255.0, 214 / 255.0));

            //Shader homer = new Phong(new Vector4(214 / 255.0, 82 / 255.0, 82 / 255.0))
            //{
            //    DiffuseSampler = new TextureSampler("Textures/homer.png", null, false)
            //    {
            //        Mapping = new PlaneMapping(new Vector4(-5, -5, 5), 10 * Vector4.UnitY,  -10 * Vector4.UnitZ)
            //    }
            //};

            //Shader rocks = new Phong(new Vector4(0, 1, 1), new Vector4(0, 0, 0))
            //{
            //    DiffuseSampler = new TextureSampler("Textures/wall-diffuse.png", null, true)
            //    {
            //        Mapping = new PlaneMapping(new Vector4(-5, -5, 5), 7 * Vector4.UnitX, -7 * Vector4.UnitZ)
            //    }
            //};
            

            #endregion

            #region Models

            //world.Models.Add(new Plane(homer, new Vector4(-5, 0, 0), new Vector4(1, 0, 0)));
            //world.Models.Add(new Plane(rocks, new Vector4(0, -5, 0), new Vector4(0, 1, 0)));
            //world.Models.Add(new Plane(rocks, new Vector4(0, 5, 0), new Vector4(0, -1, 0)));
            world.Models.Add(new Plane(blue, new Vector4(0, 0, -5), new Vector4(0, 0, 1)));
            world.Models.Add(new Plane(blue, new Vector4(0, 0, 5), new Vector4(0, 0, -1)));

            #endregion

            #region Lights

            PointLight point = new PointLight()
            {
                Origin = new Vector4(5, 0, 0),
                Intensity = 0.8,
                Range = 15.0,
                LinearAttenuation = 0.7,
                QuadraticAttenuation = 0.5
            };
            world.Lights.Add(point);

            #endregion

            camera = new Camera(500, 400)
            {
                Position = new Vector4(20, 0, 0),
                Target = new Vector4(0, 0, 0),
                World = world,
                zNear = 0.01,
                zFar = 100.0,
            };
            WriteValues();
        }

        public void InitSceneNormalMapping()
        {
            //And after nearly a whole semester of hard work we can finally render ..... a scene from wolf 3D? wtffffff
            World world = new World();

            #region Shaders

            Shader blue = new Phong(new Vector4(81 / 255.0, 138 / 255.0, 214 / 255.0));

            //Shader homer = new Phong(new Vector4(214 / 255.0, 82 / 255.0, 82 / 255.0))
            //{
            //    DiffuseSampler = new TextureSampler("Textures/homer.png", null, false)
            //    {
            //        Mapping = new PlaneMapping(new Vector4(-5, -5, 5), 10 * Vector4.UnitY, -10 * Vector4.UnitZ)
            //    }
            //};

            //Shader rocksLeft = new Phong(new Vector4(0, 1, 1), new Vector4(0.5, 0.5, 0.5))
            //{
            //    DiffuseSampler = new TextureSampler("Textures/wall-diffuse.png", null, true)
            //    {
            //        Mapping = new PlaneMapping(new Vector4(-5, -5, 5), 7 * Vector4.UnitX, -7 * Vector4.UnitZ)
            //    },

            //    NormalSampler = new NormalSampler("Textures/wall-normal.png", null, true)
            //    {
            //        Mapping = new PlaneMapping(new Vector4(-5, -5, 5), 7 * Vector4.UnitX, -7 * Vector4.UnitZ)
            //    }
            //};

            //Shader rocksRight = new Phong(new Vector4(0, 1, 1), new Vector4(0.5, 0.5, 0.5))
            //{
            //    DiffuseSampler = new TextureSampler("Textures/wall-diffuse.png", null, true)
            //    {
            //        Mapping = new PlaneMapping(new Vector4(-5, 5, 5), 7 * Vector4.UnitX, -7 * Vector4.UnitZ)
            //    },

            //    NormalSampler = new NormalSampler("Textures/wall-normal.png", null, true)
            //    {
            //        Mapping = new PlaneMapping(new Vector4(-5, 5, 5), 7 * Vector4.UnitX, -7 * Vector4.UnitZ)
            //    }
            //};

            #endregion

            #region Models

            //world.Models.Add(new Plane(homer, new Vector4(-5, 0, 0), new Vector4(1, 0, 0)));
            //world.Models.Add(new Plane(rocksLeft, new Vector4(0, -5, 0), new Vector4(0, 1, 0)));
            //world.Models.Add(new Plane(rocksRight, new Vector4(0, 5, 0), new Vector4(0, -1, 0)));
            world.Models.Add(new Plane(blue, new Vector4(0, 0, -5), new Vector4(0, 0, 1)));
            world.Models.Add(new Plane(blue, new Vector4(0, 0, 5), new Vector4(0, 0, -1)));

           

            #endregion

            #region Lights

            PointLight point = new PointLight()
            {
                Origin = new Vector4(5, 0, 0),
                Intensity = 0.8,
                Range = 15.0,
                LinearAttenuation = 0.7,
                QuadraticAttenuation = 0.5
            };
            world.Lights.Add(point);

            #endregion

            camera = new Camera(500, 400)
            {
                Position = new Vector4(20, 0, 0),
                Target = new Vector4(0, 0, 0),
                World = world,
                zNear = 0.01,
                zFar = 100.0,
            };
            WriteValues();
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            Graphics g = e.Graphics;
            this.Width = camera.BitmapWidth + 240;
            this.Height = camera.BitmapHeight + 120;
            g.DrawImage(camera.Bitmap, 0, 0);
        }

        Double Parse(String text)
        {
            NumberStyles styles = NumberStyles.Integer | NumberStyles.AllowDecimalPoint;
            double res = 0;
            float rr = 0;
            double.TryParse(text, styles, provider, out res);
            float.TryParse(text, styles, provider, out rr);
            return res;
        }

        private void bRender_Click(object sender, EventArgs e)
        {
            Cursor = Cursors.WaitCursor;
            DateTime t0 = DateTime.Now;
            camera.UseShadows = cbUseShadows.Checked;

            ReadValues();

            camera.Render();

            DateTime t1 = DateTime.Now;
            Cursor = Cursors.Default;
            lRenderTime.Text = "Rendering: " + (t1 - t0).TotalSeconds.ToString("F3") + " s";
            Invalidate();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            SaveFileDialog sfd = new SaveFileDialog();
            sfd.Filter = "Images|*.png;*.bmp;*.jpg";
            sfd.InitialDirectory = System.IO.Directory.GetCurrentDirectory() + "\\Output";
            System.Drawing.Imaging.ImageFormat format = System.Drawing.Imaging.ImageFormat.Png;
            if (sfd.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                camera.Bitmap.Save(sfd.FileName, format);
        }

        #region Read & Write

        private void ReadValues()
        {
            camera.FovY = Parse(textBox1.Text);
            camera.zNear = Parse(textBox8.Text);
            camera.zFar = Parse(textBox9.Text);

            camera.BitmapWidth = (int)Parse(textBox11.Text);
            camera.BitmapHeight = (int)Parse(textBox10.Text);

            camera.Position.X = Parse(textBox2.Text);
            camera.Position.Y = Parse(textBox3.Text);
            camera.Position.Z = Parse(textBox4.Text);

            camera.Target.X = Parse(textBox5.Text);
            camera.Target.Y = Parse(textBox6.Text);
            camera.Target.Z = Parse(textBox7.Text);
        }

        private void WriteValues()
        {
            textBox1.Text = camera.FovY.ToString(provider);
            textBox8.Text = camera.zNear.ToString(provider);
            textBox9.Text = camera.zFar.ToString(provider);

            textBox11.Text = camera.BitmapWidth.ToString(provider);
            textBox10.Text = camera.BitmapHeight.ToString(provider);

            textBox2.Text = camera.Position.X.ToString(provider);
            textBox3.Text = camera.Position.Y.ToString(provider);
            textBox4.Text = camera.Position.Z.ToString(provider);

            textBox5.Text = camera.Target.X.ToString(provider);
            textBox6.Text = camera.Target.Y.ToString(provider);
            textBox7.Text = camera.Target.Z.ToString(provider);
        }

        #endregion

        private void cbScene_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch ((String)cbScene.SelectedItem)
            {
                case "Sphere Mapping": InitSceneSphereMapping(); break;
                case "Plane Mapping": InitScenePlaneMapping(); break;
                case "Normal Mapping": InitSceneNormalMapping(); break;
            }
        }

        private void cbAntiAlias_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch ((String)cbAntiAlias.SelectedItem)
            {
                case "No AntiAliasing": camera.SubSampleCount = 1; break;
                case "2x2 AntiAliasing": camera.SubSampleCount = 2; break;
                case "4x4 AntiAliasing": camera.SubSampleCount = 4; break;
                case "8x8 AntiAliasing": camera.SubSampleCount = 8; break;
            }   
        }
    }
}
